#include <vector>
using std::vector;

#include "vec3.hpp"
#include "trajectory.hpp"

vector<trajectory> bundle;
const double dt = 0.001;

void Euler(trajectory &T, vec3 (&F)(const trajectory&), double dt)
{
	T.increment_and_log(F(T)*dt);
}

void RungaKutta2nd(trajectory &T, vec3 (&F)(const trajectory&), double dt)
{
	const vec3 c1 = F(T)*0.5*dt;
	T.increment(c1);              // moves particle to r + c1
	const vec3 c2 = F(T)*dt;
	T.increment_and_log(c2-c1);   // moves particle to r + c2
}

void RungaKutta4th(trajectory &T, vec3 (&F)(const trajectory&), double dt)
{
	const vec3 c1 = F(T)*dt;
	T.increment(c1*0.5);       // moves particle to r + c1/2
	const vec3 c2 = F(T)*dt;
	T.increment((c2-c1)*0.5);  // moves particle to r + c2/2
	const vec3 c3 = F(T)*dt;
	T.increment(c3-c2*0.5);    // moves particle to r + c3;
	const vec3 c4 = F(T)*dt;
	T.increment_and_log((c1+c2*2.0+c3*2.0+c4)/6.0-c3); // moves particle to r + (c1+2c2+2c3+c4)/6
}

vec3 Roessler(const trajectory &T)
{
	const double a = 0.1;
	const double b = 0.1;
	const double c = 14.0;
	return vec3( - T.y() - T.z(), T.x() + a*T.y(), b + T.z()*(T.x() - c) );
}

vec3 Lorenz(const trajectory &T)
{
	const double sigma = 10.0;
	const double beta = 8.0/3.0;
	const double rho = 99.96; // decay<24.74, chaos>24.74, regular orbits=99.96,160
	return vec3( sigma*(T.y() - T.x()), T.x()*(rho - T.z()) - T.y(), T.x()*T.y() - beta*T.z() );
}

void initialize_bundle(void)
{
	const double eps = 0.01;
	const double x0 = 0.2;
	const double y0 = 0.4;
	const double z0 = 0.6;
	
	bundle.push_back(trajectory(x0,y0,z0));
	bundle.push_back(trajectory(x0+eps,y0,z0));
	bundle.push_back(trajectory(x0-eps,y0,z0));
	bundle.push_back(trajectory(x0,y0+eps,z0));
	bundle.push_back(trajectory(x0,y0-eps,z0));
	bundle.push_back(trajectory(x0,y0,z0+eps));
	bundle.push_back(trajectory(x0,y0,z0-eps));
}

void update_bundle(void)
{
	for (size_t n = 0; n < bundle.size(); ++n)
		bundle[n].evolve(Lorenz,RungaKutta4th,dt);

}

void draw_bundle(void)
{
	for (size_t n = 0; n < bundle.size(); ++n)
		bundle[n].draw();
}
